@@ -3,45 +3,44 @@ import hashlib
 import hmac
 import json
 import os
-from typing import Any, Dict, Tuple, Union
+from decimal import Decimal
+from typing import Any, Dict, Union
 
 from Cryptodome.Cipher import AES
-from nacl.bindings import crypto_scalarmult
-from nacl.signing import SigningKey
-from pytoniq_core import Address, Cell, MessageAny, begin_cell, HashMap
+from nacl.bindings import (
+    crypto_scalarmult,
+    crypto_sign_ed25519_sk_to_curve25519,
+    crypto_sign_ed25519_pk_to_curve25519,
+)
+from pytoniq_core import Address, Cell, MessageAny, begin_cell, HashMap, Slice
 
 
-def message_to_boc_hex(message: MessageAny, normalize_hash: bool = True) -> Tuple[str, str]:
+def normalize_hash(message: Union[MessageAny, str]) -> bytes:
     """
-    Converts a message into its BoC (Bag of Cells) hex representation and computes a hash.
+    Computes the normalized hash of an external message.
+    If the message is not external, returns the regular hash.
 
-    If `normalize_hash` is True, the hash is computed on a normalized version of the message:
-    - Only the destination address and body are retained.
-    - The body is stored as a reference in a new cell for consistent structure and hash.
-
-    :param message: The original message object to serialize.
-    :param normalize_hash: Whether to normalize the message before hashing.
-    :return: A tuple containing (BoC hex string, message hash hex string).
+    :param message: The message object or its BoC string.
+    :return: Hash as raw bytes.
     """
-    # Serialize the full message into a cell and convert to BoC hex
-    message_cell = message.serialize()
-    message_boc = message_cell.to_boc()
+    if isinstance(message, str):
+        message = MessageAny.deserialize(Slice.one_from_boc(message))
 
-    # Compute the hash from either the original or normalized message
-    if normalize_hash and message.is_external:
-        message_cell = (
-            begin_cell()
-            .store_uint(2, 2)
-            .store_address(None)
-            .store_address(message.info.dest)
-            .store_coins(0)
-            .store_bool(False)
-            .store_bool(True)
-            .store_ref(message.body)
-            .end_cell()
-        )
+    if not message.is_external:
+        return message.serialize().hash
 
-    return message_boc.hex(), message_cell.hash.hex()
+    cell = (
+        begin_cell()
+        .store_uint(2, 2)
+        .store_address(None)
+        .store_address(message.info.dest)
+        .store_coins(0)
+        .store_bool(False)
+        .store_bool(True)
+        .store_ref(message.body)
+        .end_cell()
+    )
+    return cell.hash
 
 
 def boc_to_base64_string(boc: Union[str, bytes]) -> str:
@@ -79,17 +78,17 @@ def create_encrypted_comment_cell(
     """
     root = begin_cell().store_uint(0x2167da4b, 32)
 
-    our_private_key_bytes = our_private_key[:32]
+    our_public_key_bytes = our_private_key[32:]
     their_public_key_bytes = their_public_key.to_bytes(32, byteorder='big')
 
     # Convert keys to Curve25519
-    _our_private_key = SigningKey(our_private_key_bytes).to_curve25519_private_key().encode()
-    _their_public_key = SigningKey(their_public_key_bytes).verify_key.to_curve25519_public_key().encode()
+    _our_private_key = crypto_sign_ed25519_sk_to_curve25519(our_private_key)
+    _their_public_key = crypto_sign_ed25519_pk_to_curve25519(their_public_key_bytes)
 
     # Compute shared key
     shared_key = crypto_scalarmult(_our_private_key, _their_public_key)
 
-    data = text.encode('utf-8')
+    data = text.encode("utf-8")
 
     # Calculate prefix size and generate random prefix
     pfx_sz = 16
@@ -101,7 +100,7 @@ def create_encrypted_comment_cell(
     data = bytes(pfx) + data
 
     # Generate message key using HMAC with sender address
-    h = hmac.new(sender_address.to_str().encode('utf-8'), data, hashlib.sha512)
+    h = hmac.new(sender_address.to_str().encode("utf-8"), data, hashlib.sha512)
     msg_key = h.digest()[:16]
 
     # Generate encryption key using HMAC with shared key
@@ -113,9 +112,7 @@ def create_encrypted_comment_cell(
     encrypted_data = c.encrypt(data)
 
     # XOR public keys
-    xor_key = bytearray(SigningKey(our_private_key_bytes).verify_key.encode())
-    for i in range(32):
-        xor_key[i] ^= SigningKey(their_public_key_bytes).verify_key.encode()[i]
+    xor_key = bytes([a ^ b for a, b in zip(our_public_key_bytes, their_public_key_bytes)])
 
     # Store data
     root.store_bytes(xor_key)
@@ -146,7 +143,7 @@ def to_amount(value: int, decimals: int = 9, precision: int = 2) -> Union[float,
     return rounded_ton_value if rounded_ton_value % 1 != 0 else int(rounded_ton_value)
 
 
-def to_nano(value: Union[int, float], decimals: int = 9) -> int:
+def to_nano(value: Union[int, float, Decimal], decimals: int = 9) -> int:
     """
     Converts TON value to nanoton.
 
@@ -154,7 +151,7 @@ def to_nano(value: Union[int, float], decimals: int = 9) -> int:
     :param decimals: The number of decimal places in the input value. Defaults to 9.
     :return: The value of the input in nanoton.
     """
-    if not isinstance(value, (int, float)):
+    if not isinstance(value, (int, float, Decimal)):
         raise ValueError("Value must be a positive integer or float.")
 
     return round(value * (10 ** decimals))
@@ -182,3 +179,29 @@ def serialize_onchain_dict(data: Dict[str, Any]) -> Cell:
         dict_cell.set(key, cell.end_cell(), hash_key=True)
 
     return dict_cell.serialize()
+
+
+def cell_hash(c: Cell) -> int:
+    """
+    Calculates the representation hash of the given cell c and returns it as a 256-bit unsigned integer x.
+    This function is handy for signing and verifying signatures of arbitrary entities structured as a tree of cells.
+    """
+    return int.from_bytes(c.hash, "big")
+
+
+def slice_hash(s: Slice) -> int:
+    """
+    Computes the hash of the given slice s and returns it as a 256-bit unsigned integer x.
+    The result is equivalent to creating a standard cell containing only the data and references from s and then computing its hash using cell_hash.
+    """
+    return int.from_bytes(s.to_cell().hash, "big")
+
+
+def string_hash(s: str) -> int:
+    """
+    Calculates the SHA-256 hash of the data bits in the given slice s.
+    A cell underflow exception is thrown if the bit length of s is not a multiple of eight.
+    The hash is returned as a 256-bit unsigned integer x.
+    """
+    data = begin_cell().store_snake_string(s).end_cell().get_data_bytes()
+    return int.from_bytes(hashlib.sha256(data).digest(), "big")
