@@ -5,6 +5,8 @@ from .contract import (
     HighloadWalletV3,
     PreprocessedWalletV2,
     PreprocessedWalletV2R1,
+    WalletV2R1,
+    WalletV2R2,
     WalletV3R1,
     WalletV3R2,
     WalletV4R1,
@@ -19,6 +21,8 @@ __all__ = [
     "HighloadWalletV3",
     "PreprocessedWalletV2",
     "PreprocessedWalletV2R1",
+    "WalletV2R1",
+    "WalletV2R2",
     "WalletV3R1",
     "WalletV3R2",
     "WalletV4R1",
