@@ -1,7 +1,7 @@
 from ._base import Wallet
 
 from .highload import HighloadWalletV2, HighloadWalletV3
-from .prepocessed import PreprocessedWalletV2, PreprocessedWalletV2R1
+from .preprocessed import PreprocessedWalletV2, PreprocessedWalletV2R1
 from .v2 import WalletV2R1, WalletV2R2
 from .v3 import WalletV3R1, WalletV3R2
 from .v4 import WalletV4R1, WalletV4R2
