@@ -2,7 +2,7 @@ TEXT_COMMENT_OPCODE = 0
 
 SIGNED_EXTERNAL_OPCODE = 0x7369676e
 
-SIGNED_ITERNAL_OPCODE = 0x73696e74
+SIGNED_INTERNAL_OPCODE = 0x73696e74
 
 INTERNAL_TRANSFER_OPCODE = 0xae42e5a4
 
